@@ -265,6 +265,15 @@ $VERSION = '1.65';
         'XMP',
         'ZoomedPreviewImage',
     ],
+    'ls-l' => [
+        'FilePermissions',
+        'FileHardLinks',
+        'FileUserID',
+        'FileGroupID',
+        'FileSize#',
+        'FileModifyDate',
+        'FileName',
+    ],
 );
 
 #------------------------------------------------------------------------------
@@ -332,7 +341,7 @@ name prefix.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
